@@ -1,7 +1,9 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 //
 //  file:  rbbirb.cpp
 //
-//  Copyright (C) 2002-2003, International Business Machines Corporation and others.
+//  Copyright (C) 2002-2011, International Business Machines Corporation and others.
 //  All Rights Reserved.
 //
 //  This file contains the RBBIRuleBuilder class implementation.  This is the main class for
@@ -22,16 +24,16 @@
 #include "unicode/uchriter.h"
 #include "unicode/parsepos.h"
 #include "unicode/parseerr.h"
+
 #include "cmemory.h"
 #include "cstring.h"
-
 #include "rbbirb.h"
 #include "rbbinode.h"
-
 #include "rbbiscan.h"
 #include "rbbisetb.h"
 #include "rbbitblb.h"
 #include "rbbidata.h"
+#include "uassert.h"
 
 
 U_NAMESPACE_BEGIN
@@ -43,12 +45,12 @@ U_NAMESPACE_BEGIN
 //
 //----------------------------------------------------------------------------------------
 RBBIRuleBuilder::RBBIRuleBuilder(const UnicodeString   &rules,
-                                       UParseError     &parseErr,
+                                       UParseError     *parseErr,
                                        UErrorCode      &status)
  : fRules(rules)
 {
     fStatus = &status; // status is checked below
-    fParseError = &parseErr;
+    fParseError = parseErr;
     fDebugEnv   = NULL;
 #ifdef RBBI_DEBUG
     fDebugEnv   = getenv("U_RBBIDEBUG");
@@ -64,22 +66,30 @@ RBBIRuleBuilder::RBBIRuleBuilder(const UnicodeString   &rules,
     fReverseTables      = NULL;
     fSafeFwdTables      = NULL;
     fSafeRevTables      = NULL;
+    fRuleStatusVals     = NULL;
     fChainRules         = FALSE;
     fLBCMNoChain        = FALSE;
     fLookAheadHardBreak = FALSE;
-
-    UErrorCode oldstatus = status;   
-
-    fUSetNodes          = new UVector(status); // bcos status gets overwritten here
-    fScanner            = new RBBIRuleScanner(this);
-    fSetBuilder         = new RBBISetBuilder(this);
-    if (U_FAILURE(oldstatus)) {
-        status = oldstatus;
+    fUSetNodes          = NULL;
+    fRuleStatusVals     = NULL;
+    fScanner            = NULL;
+    fSetBuilder         = NULL;
+    if (parseErr) {
+        uprv_memset(parseErr, 0, sizeof(UParseError));
     }
+
     if (U_FAILURE(status)) {
         return;
     }
-    if(fSetBuilder == 0 || fScanner == 0 || fUSetNodes == 0) {
+
+    fUSetNodes          = new UVector(status); // bcos status gets overwritten here
+    fRuleStatusVals     = new UVector(status);
+    fScanner            = new RBBIRuleScanner(this);
+    fSetBuilder         = new RBBISetBuilder(this);
+    if (U_FAILURE(status)) {
+        return;
+    }
+    if(fSetBuilder == 0 || fScanner == 0 || fUSetNodes == 0 || fRuleStatusVals == 0) {
         status = U_MEMORY_ALLOCATION_ERROR;
     }
 }
@@ -114,6 +124,7 @@ RBBIRuleBuilder::~RBBIRuleBuilder() {
     delete fSafeFwdTree;
     delete fSafeRevTree;
     delete fScanner;
+    delete fRuleStatusVals;
 }
 
 
@@ -130,12 +141,14 @@ RBBIRuleBuilder::~RBBIRuleBuilder() {
 static int32_t align8(int32_t i) {return (i+7) & 0xfffffff8;}
 
 RBBIDataHeader *RBBIRuleBuilder::flattenData() {
+    int32_t    i;
+
     if (U_FAILURE(*fStatus)) {
         return NULL;
     }
 
     // Remove comments and whitespace from the rules to make it smaller.
-    UnicodeString strippedRules(RBBIRuleScanner::stripRules(fRules));
+    UnicodeString strippedRules((const UnicodeString&)RBBIRuleScanner::stripRules(fRules));
 
     // Calculate the size of each section in the data.
     //   Sizes here are padded up to a multiple of 8 for better memory alignment.
@@ -148,10 +161,18 @@ RBBIDataHeader *RBBIRuleBuilder::flattenData() {
     int32_t safeFwdTableSize  = align8(fSafeFwdTables->getTableSize());
     int32_t safeRevTableSize  = align8(fSafeRevTables->getTableSize());
     int32_t trieSize          = align8(fSetBuilder->getTrieSize());
+    int32_t statusTableSize   = align8(fRuleStatusVals->size() * sizeof(int32_t));
     int32_t rulesSize         = align8((strippedRules.length()+1) * sizeof(UChar));
 
-    int32_t         totalSize = headerSize + forwardTableSize + reverseTableSize
-                                + safeFwdTableSize + safeRevTableSize + trieSize + rulesSize;
+    (void)safeFwdTableSize;
+
+    int32_t         totalSize = headerSize
+                                + forwardTableSize 
+                                + /* reverseTableSize */ 0
+                                + /* safeFwdTableSize */ 0
+                                + (safeRevTableSize ? safeRevTableSize : reverseTableSize)
+                                + statusTableSize + trieSize + rulesSize;
+
     RBBIDataHeader  *data     = (RBBIDataHeader *)uprv_malloc(totalSize);
     if (data == NULL) {
         *fStatus = U_MEMORY_ALLOCATION_ERROR;
@@ -160,32 +181,70 @@ RBBIDataHeader *RBBIRuleBuilder::flattenData() {
     uprv_memset(data, 0, totalSize);
 
 
-    data->fMagic         = 0xb1a0;
-    data->fVersion       = 1;
-    data->fLength        = totalSize;
-    data->fCatCount      = fSetBuilder->getNumCharCategories();
+    data->fMagic            = 0xb1a0;
+    data->fFormatVersion[0] = RBBI_DATA_FORMAT_VERSION[0];
+    data->fFormatVersion[1] = RBBI_DATA_FORMAT_VERSION[1];
+    data->fFormatVersion[2] = RBBI_DATA_FORMAT_VERSION[2];
+    data->fFormatVersion[3] = RBBI_DATA_FORMAT_VERSION[3];
+    data->fLength           = totalSize;
+    data->fCatCount         = fSetBuilder->getNumCharCategories();
+
+    // Only save the forward table and the safe reverse table,
+    // because these are the only ones used at run-time.
+    //
+    // For the moment, we still build the other tables if they are present in the rule source files,
+    // for backwards compatibility. Old rule files need to work, and this is the simplest approach.
+    //
+    // Additional backwards compatibility consideration: if no safe rules are provided, consider the
+    // reverse rules to actually be the safe reverse rules.
 
     data->fFTable        = headerSize;
     data->fFTableLen     = forwardTableSize;
-    data->fRTable        = data->fFTable  + forwardTableSize;
-    data->fRTableLen     = reverseTableSize;
-    data->fSFTable       = data->fRTable  + reverseTableSize;
-    data->fSFTableLen    = safeFwdTableSize;
-    data->fSRTable       = data->fSFTable + safeFwdTableSize;
-    data->fSRTableLen    = safeRevTableSize;
 
-    data->fTrie          = data->fSRTable + safeRevTableSize;
+    // Do not save Reverse Table.
+    data->fRTable        = data->fFTable  + forwardTableSize;
+    data->fRTableLen     = 0;
+
+    // Do not save the Safe Forward table.
+    data->fSFTable       = data->fRTable + 0;
+    data->fSFTableLen    = 0;
+
+    data->fSRTable       = data->fSFTable + 0;
+    if (safeRevTableSize > 0) {
+        data->fSRTableLen    = safeRevTableSize;
+    } else if (reverseTableSize > 0) {
+        data->fSRTableLen    = reverseTableSize;
+    } else {
+        U_ASSERT(FALSE);    // Rule build should have failed for lack of a reverse table
+                            // before reaching this point.
+    }
+        
+
+    data->fTrie          = data->fSRTable + data->fSRTableLen;
     data->fTrieLen       = fSetBuilder->getTrieSize();
-    data->fRuleSource    = data->fTrie    + trieSize;
+    data->fStatusTable   = data->fTrie    + trieSize;
+    data->fStatusTableLen= statusTableSize;
+    data->fRuleSource    = data->fStatusTable + statusTableSize;
     data->fRuleSourceLen = strippedRules.length() * sizeof(UChar);
 
     uprv_memset(data->fReserved, 0, sizeof(data->fReserved));
 
     fForwardTables->exportTable((uint8_t *)data + data->fFTable);
-    fReverseTables->exportTable((uint8_t *)data + data->fRTable);
-    fSafeFwdTables->exportTable((uint8_t *)data + data->fSFTable);
-    fSafeRevTables->exportTable((uint8_t *)data + data->fSRTable);
+    // fReverseTables->exportTable((uint8_t *)data + data->fRTable);
+    // fSafeFwdTables->exportTable((uint8_t *)data + data->fSFTable);
+    if (safeRevTableSize > 0) {
+        fSafeRevTables->exportTable((uint8_t *)data + data->fSRTable);
+    } else {
+        fReverseTables->exportTable((uint8_t *)data + data->fSRTable);
+    }
+
     fSetBuilder->serializeTrie ((uint8_t *)data + data->fTrie);
+
+    int32_t *ruleStatusTable = (int32_t *)((uint8_t *)data + data->fStatusTable);
+    for (i=0; i<fRuleStatusVals->size(); i++) {
+        ruleStatusTable[i] = fRuleStatusVals->elementAti(i);
+    }
+
     strippedRules.extract((UChar *)((uint8_t *)data+data->fRuleSource), rulesSize/2+1, *fStatus);
 
     return data;
@@ -204,7 +263,7 @@ RBBIDataHeader *RBBIRuleBuilder::flattenData() {
 //----------------------------------------------------------------------------------------
 BreakIterator *
 RBBIRuleBuilder::createRuleBasedBreakIterator( const UnicodeString    &rules,
-                                    UParseError      &parseError,
+                                    UParseError      *parseError,
                                     UErrorCode       &status)
 {
     // status checked below
@@ -214,10 +273,10 @@ RBBIRuleBuilder::createRuleBasedBreakIterator( const UnicodeString    &rules,
     // and list of all Unicode Sets referenced by the rules.
     //
     RBBIRuleBuilder  builder(rules, parseError, status);
-    builder.fScanner->parse();
     if (U_FAILURE(status)) { // status checked here bcos build below doesn't
         return NULL;
     }
+    builder.fScanner->parse();
 
     //
     // UnicodeSet processing.
@@ -235,11 +294,14 @@ RBBIRuleBuilder::createRuleBasedBreakIterator( const UnicodeString    &rules,
     builder.fReverseTables = new RBBITableBuilder(&builder, &builder.fReverseTree);
     builder.fSafeFwdTables = new RBBITableBuilder(&builder, &builder.fSafeFwdTree);
     builder.fSafeRevTables = new RBBITableBuilder(&builder, &builder.fSafeRevTree);
-    if (U_SUCCESS(status)
-        && (builder.fForwardTables == NULL || builder.fReverseTables == NULL ||
-            builder.fSafeFwdTables == NULL || builder.fSafeRevTables == NULL)) 
+    if (builder.fForwardTables == NULL || builder.fReverseTables == NULL ||
+        builder.fSafeFwdTables == NULL || builder.fSafeRevTables == NULL)
     {
         status = U_MEMORY_ALLOCATION_ERROR;
+        delete builder.fForwardTables; builder.fForwardTables = NULL;
+        delete builder.fReverseTables; builder.fReverseTables = NULL;
+        delete builder.fSafeFwdTables; builder.fSafeFwdTables = NULL;
+        delete builder.fSafeRevTables; builder.fSafeRevTables = NULL;
         return NULL;
     }
 
@@ -247,15 +309,21 @@ RBBIRuleBuilder::createRuleBasedBreakIterator( const UnicodeString    &rules,
     builder.fReverseTables->build();
     builder.fSafeFwdTables->build();
     builder.fSafeRevTables->build();
-    if (U_FAILURE(status)) {
-        return NULL;
+
+#ifdef RBBI_DEBUG
+    if (builder.fDebugEnv && uprv_strstr(builder.fDebugEnv, "states")) {
+        builder.fForwardTables->printRuleStatusTable();
     }
+#endif
 
     //
     //   Package up the compiled data into a memory image
     //      in the run-time format.
     //
     RBBIDataHeader *data = builder.flattenData(); // returns NULL if error
+    if (U_FAILURE(*builder.fStatus)) {
+        return NULL;
+    }
 
 
     //
