@@ -1,6 +1,12 @@
-/**************************************************************************
+/*************************************************************************
 *
-*   Copyright (C) 2000-2011, International Business Machines
+*   © 2016 and later: Unicode, Inc. and others.
+*   License & terms of use: http://www.unicode.org/copyright.html#License
+*
+**************************************************************************
+**************************************************************************
+*
+*   Copyright (C) 2000-2016, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ***************************************************************************
@@ -43,6 +49,9 @@
 #include "flagcb.h"
 
 /* Some utility functions */
+#ifndef UPRV_LENGTHOF
+#define UPRV_LENGTHOF(array) (int32_t)(sizeof(array)/sizeof((array)[0]))
+#endif
 
 static const UChar kNone[] = { 0x0000 };
 
@@ -58,20 +67,13 @@ void prettyPrintUChar(UChar c)
     char buf[1000];
     UErrorCode status = U_ZERO_ERROR;
     int32_t o;
-    
-    o = u_charName(c, U_UNICODE_CHAR_NAME, buf, 1000, &status);
+
+    o = u_charName(c, U_EXTENDED_CHAR_NAME, buf, 1000, &status);
     if(U_SUCCESS(status) && (o>0) ) {
       buf[6] = 0;
       printf("%7s", buf);
     } else {
-      o = u_charName(c, U_UNICODE_10_CHAR_NAME, buf, 1000, &status);
-      if(U_SUCCESS(status) && (o>0)) {
-        buf[5] = 0;
-        printf("~%6s", buf);
-      }
-      else {
-        printf(" ??????");
-      }
+      printf(" ??????");
     }
   } else {
     switch((char)(c & 0x007F)) {
@@ -212,7 +214,9 @@ UErrorCode convsample_02()
   int32_t     len;
 
   // set up the converter
+  //! [ucnv_open]
   conv = ucnv_open("koi8-r", &status);
+  //! [ucnv_open]
   assert(U_SUCCESS(status));
 
   // convert to koi8-r
@@ -791,8 +795,7 @@ UBool convsample_21_didSubstitute(const char *source)
          debugCtx1->subContext, flagCtx, debugCtx2, debugCtx2->subCallback);
 #endif
 
-  cloneLen = 1; /* but passing in null so it will clone */
-  cloneCnv = ucnv_safeClone(conv,  NULL,  &cloneLen, &status);
+  cloneCnv = ucnv_safeClone(conv, NULL, NULL, &status);
 
   U_ASSERT(status);
 
@@ -1073,6 +1076,42 @@ UErrorCode convsample_46()
 
 #define BUFFERSIZE 219
 
+void convsample_50() {
+  printf("\n\n==============================================\n"
+         "Sample 50: C: ucnv_detectUnicodeSignature\n");
+
+  //! [ucnv_detectUnicodeSignature]
+  UErrorCode err = U_ZERO_ERROR;
+  UBool discardSignature = TRUE; /* set to TRUE to throw away the initial U+FEFF */
+  char input[] = { '\xEF','\xBB', '\xBF','\x41','\x42','\x43' };
+  int32_t signatureLength = 0;
+  const char *encoding = ucnv_detectUnicodeSignature(input,sizeof(input),&signatureLength,&err);
+  UConverter *conv = NULL;
+  UChar output[100];
+  UChar *target = output, *out;
+  const char *source = input;
+  if(encoding!=NULL && U_SUCCESS(err)){
+    // should signature be discarded ?
+    conv = ucnv_open(encoding, &err);
+    // do the conversion
+    ucnv_toUnicode(conv,
+                   &target, output + UPRV_LENGTHOF(output),
+                   &source, input + sizeof(input),
+                   NULL, TRUE, &err);
+    out = output;
+    if (discardSignature){
+      ++out; // ignore initial U+FEFF
+    }
+    while(out != target) {
+      printf("%04x ", *out++);
+    }
+    puts("");
+  }
+  //! [ucnv_detectUnicodeSignature]
+  puts("");
+}
+
+
 
 /* main */
 
@@ -1096,6 +1135,8 @@ int main()
   convsample_40();  // C,   cp37 -> UTF16 [data02.bin -> data40.utf16]
   
   convsample_46();  // C,  UTF16 -> latin3 [data41.utf16 -> data46.out]
+
+  convsample_50();  // C, detect unicode signature
   
   printf("End of converter samples.\n");
   
