@@ -1,7 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
- * COPYRIGHT: 
- * Copyright (c) 1997-2005, International Business Machines Corporation and
- * others. All Rights Reserved.
+ * Copyright (c) 1997-2013 International Business Machines 
+ * Corporation and others. All Rights Reserved.
  ********************************************************************/
 /********************************************************************************
 *
@@ -10,6 +11,7 @@
 * Modification History:
 *        Name                     Description            
 *     Madhu Katragadda            Converted to C
+*     Brian Rower                 Added TestOpenVsOpenRules
 *********************************************************************************
 *//* C API TEST For COLLATOR */
 
@@ -77,6 +79,11 @@
      **/
     void TestCloneBinary(void);
 
+    /**
+     * Test ucol_open() vs. ucol_openRules()
+     **/
+    void TestOpenVsOpenRules(void);
+
     /**
      * Test getting bounds for a sortkey
      */
@@ -105,12 +112,6 @@
      */
     void TestMergeSortKeys(void);
 
-    /**
-     * utility function, defined in cmsccoll.c
-     */
-    void genericLocaleStarter(const char *locale, const char *s[], uint32_t size);
-
-
     /** 
      * test short string and collator identifier functions
      */
@@ -125,6 +126,17 @@
      * Test funny stuff with open binary
      */
     static void TestOpenBinary(void);
+
+    /**
+     * Test getKeywordValuesForLocale API
+     */
+    static void TestGetKeywordValuesForLocale(void);
+
+    /**
+     * test strcoll with null arg
+     */
+    static void TestStrcollNull(void);
+
 #endif /* #if !UCONFIG_NO_COLLATION */
 
 #endif
