@@ -1,12 +1,14 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 *******************************************************************************
 *
-*   Copyright (C) 2007-2014, International Business Machines
+*   Copyright (C) 2007-2016, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 *******************************************************************************
 *   file name:  udatpg_test.c
-*   encoding:   US-ASCII
+*   encoding:   UTF-8
 *   tab size:   8 (not used)
 *   indentation:4
 *
@@ -30,6 +32,7 @@
 #include "unicode/udatpg.h"
 #include "unicode/ustring.h"
 #include "cintltst.h"
+#include "cmemory.h"
 
 void addDateTimePatternGeneratorTest(TestNode** root);
 
@@ -217,10 +220,10 @@ static void TestUsage() {
     }
     
     /* set append name to hr */
-    udatpg_setAppendItemName( dtpg, UDATPG_HOUR_FIELD, appendItemName, 7 );
+    udatpg_setAppendItemName(dtpg, UDATPG_HOUR_FIELD, appendItemName, 2);
     r = udatpg_getAppendItemName(dtpg, UDATPG_HOUR_FIELD, &length);
     
-    if(length!=7 || 0!=u_memcmp(r, appendItemName, length) || r[length]!=0) { 
+    if(length!=2 || 0!=u_memcmp(r, appendItemName, length) || r[length]!=0) { 
         log_err("udatpg_setAppendItemName did not return the expected string\n");
         return;
     }
@@ -338,7 +341,7 @@ static void TestBuilder() {
     udatpg_close(dtpg);
     
     /* sample code in Userguide */
-    patternCapacity = (int32_t)(sizeof(pattern)/sizeof((pattern)[0]));
+    patternCapacity = UPRV_LENGTHOF(pattern);
     status=U_ZERO_ERROR;
     generator=udatpg_open(locale, &status);
     if(U_FAILURE(status)) {
@@ -357,7 +360,7 @@ static void TestBuilder() {
     }
 
     /* use it to format (or parse) */
-    formattedCapacity = (int32_t)(sizeof(formatted)/sizeof((formatted)[0]));
+    formattedCapacity = UPRV_LENGTHOF(formatted);
     resultLen=udat_format(formatter, ucal_getNow(), formatted, formattedCapacity,
                           NULL, &status);
     /* for French, the result is "13 sept." */
@@ -400,15 +403,15 @@ static void TestOptions() {
         { "en", skel_Hmm,  UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_HHcmm   },
         { "en", skel_HHmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_HHcmm   },
         { "en", skel_hhmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_hhcmm_a },
-        { "be", skel_Hmm,  UDATPG_MATCH_NO_OPTIONS,        patn_HHpmm   },
-        { "be", skel_HHmm, UDATPG_MATCH_NO_OPTIONS,        patn_HHpmm   },
-        { "be", skel_hhmm, UDATPG_MATCH_NO_OPTIONS,        patn_hpmm_a  },
-        { "be", skel_Hmm,  UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_Hpmm    },
-        { "be", skel_HHmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_HHpmm   },
-        { "be", skel_hhmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_hhpmm_a },
+        { "da", skel_Hmm,  UDATPG_MATCH_NO_OPTIONS,        patn_HHpmm   },
+        { "da", skel_HHmm, UDATPG_MATCH_NO_OPTIONS,        patn_HHpmm   },
+        { "da", skel_hhmm, UDATPG_MATCH_NO_OPTIONS,        patn_hpmm_a  },
+        { "da", skel_Hmm,  UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_Hpmm    },
+        { "da", skel_HHmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_HHpmm   },
+        { "da", skel_hhmm, UDATPG_MATCH_HOUR_FIELD_LENGTH, patn_hhpmm_a },
     };
 
-    int count = sizeof(testData) / sizeof(testData[0]);
+    int count = UPRV_LENGTHOF(testData);
     const DTPtnGenOptionsData * testDataPtr = testData;
 
     for (; count-- > 0; ++testDataPtr) {
