@@ -1,5 +1,7 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
- * COPYRIGHT: 
+ * COPYRIGHT:
  * Copyright (c) 2008-2016 International Business Machines Corporation and
  * others. All Rights Reserved.
  ********************************************************************/
@@ -18,7 +20,7 @@
  * Test basic functionality of various API functions
  **/
 class DateIntervalFormatTest: public IntlTest {
-    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );  
+    void runIndexedTest( int32_t index, UBool exec, const char* &name, char* par = NULL );
 
 public:
     /**
@@ -59,6 +61,8 @@ public:
     void testTicket11669();
     void threadFunc11669(int32_t threadNum);
 
+    void testTicket12065();
+
 private:
     /**
      * Test formatting against expected result
@@ -66,15 +70,15 @@ private:
     void expect(const char** data, int32_t data_length);
 
     /**
-     * Test formatting against expected result using user defined 
+     * Test formatting against expected result using user defined
      * DateIntervalInfo
      */
     void expectUserDII(const char** data, int32_t data_length);
 
     /**
-     * Stress test formatting 
+     * Stress test formatting
      */
-    void stress(const char** data, int32_t data_length, const Locale& loc, 
+    void stress(const char** data, int32_t data_length, const Locale& loc,
                 const char* locName);
 };
 
