@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT:
  * Copyright (c) 1997-2016 International Business Machines Corporation and
@@ -28,6 +30,7 @@ private:
     void testAllFieldPatterns(/* char* par */);
     void testStaticGetSkeleton(/* char* par */);
     void testC();
+    void testSkeletonsWithDayPeriods();
 };
 
 #endif /* #if !UCONFIG_NO_FORMATTING */
