@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /********************************************************************
  * COPYRIGHT: 
  * Copyright (c) 1997-2001, International Business Machines Corporation and
@@ -7,6 +9,9 @@
 #ifndef _INTLTESTSIMPLEDATEFORMATAPI
 #define _INTLTESTSIMPLEDATEFORMATAPI
 
+#include "unicode/utypes.h"
+
+#if !UCONFIG_NO_FORMATTING
 
 #include "intltest.h"
 
@@ -23,4 +28,6 @@ private:
     void testAPI(/* char* par */);
 };
 
+#endif /* #if !UCONFIG_NO_FORMATTING */
+
 #endif
