@@ -1,6 +1,8 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  **********************************************************************
- *   Copyright (C) 2005-2013, International Business Machines
+ *   Copyright (C) 2005-2016, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **********************************************************************
  */
@@ -11,7 +13,7 @@
 
 #include "cmemory.h"
 #include "cstring.h"
-#include "ucol_imp.h"
+#include "usrchimp.h"
 
 #include "unicode/coll.h"
 #include "unicode/tblcoll.h"
@@ -40,7 +42,6 @@ char testId[100];
     dataerrln("Failure in file %s, line %d, test ID \"%s\", status = \"%s\"", \
           __FILE__, __LINE__, testId, u_errorName(errcode));}}
 
-#define ARRAY_SIZE(array) (sizeof array / sizeof array[0])
 #define NEW_ARRAY(type, count) (type *) uprv_malloc((count) * sizeof(type))
 #define DELETE_ARRAY(array) uprv_free((void *) (array))
 
@@ -368,12 +369,10 @@ OrderList::OrderList(UCollator *coll, const UnicodeString &string, int32_t strin
     {
     default:
         strengthMask |= UCOL_TERTIARYORDERMASK;
-        /* fall through */
-
+        U_FALLTHROUGH;
     case UCOL_SECONDARY:
         strengthMask |= UCOL_SECONDARYORDERMASK;
-        /* fall through */
-
+        U_FALLTHROUGH;
     case UCOL_PRIMARY:
         strengthMask |= UCOL_PRIMARYORDERMASK;
     }
@@ -615,7 +614,7 @@ void SSearchTest::offsetTest()
         "pe\\u0302che",
     };
 
-    int32_t testCount = ARRAY_SIZE(test);
+    int32_t testCount = UPRV_LENGTHOF(test);
     UErrorCode status = U_ZERO_ERROR;
     RuleBasedCollator *col = (RuleBasedCollator *) Collator::createInstance(Locale::getEnglish(), status);
     if (U_FAILURE(status)) {
@@ -629,9 +628,6 @@ void SSearchTest::offsetTest()
     col->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
 
     for(int32_t i = 0; i < testCount; i += 1) {
-      if (i>=4 && i<=6 && logKnownIssue("9156", "was 8081")) {
-            continue; // timebomb until ticket #9156 (was #8081) is resolved
-        }
         UnicodeString ts = CharsToUnicodeString(test[i]);
         CollationElementIterator *iter = col->createCollationElementIterator(ts);
         OrderList forwardList;
@@ -752,7 +748,7 @@ void SSearchTest::sharpSTest()
                                                            &status));
     TEST_ASSERT_SUCCESS(status);
 
-    for (uint32_t t = 0; t < (sizeof(targets)/sizeof(targets[0])); t += 1) {
+    for (uint32_t t = 0; t < UPRV_LENGTHOF(targets); t += 1) {
         UBool bFound;
         UnicodeString target = targets[t].unescape();
 
@@ -1396,12 +1392,12 @@ void SSearchTest::monkeyTest(char *params)
         &expansionMonkey,
         &contractionMonkey,
         &expansionMonkey};
-    int32_t monkeyCount = sizeof(monkeys) / sizeof(monkeys[0]);
+    int32_t monkeyCount = UPRV_LENGTHOF(monkeys);
     // int32_t nonMatchCount = 0;
 
     UCollationStrength strengths[] = {UCOL_PRIMARY, UCOL_SECONDARY, UCOL_TERTIARY};
     const char *strengthNames[] = {"primary", "secondary", "tertiary"};
-    int32_t strengthCount = sizeof(strengths) / sizeof(strengths[0]);
+    int32_t strengthCount = UPRV_LENGTHOF(strengths);
     int32_t loopCount = quick? 1000 : 10000;
     int32_t firstStrength = 0;
     int32_t lastStrength  = strengthCount - 1; //*/ 0;
@@ -1449,7 +1445,7 @@ void SSearchTest::monkeyTest(char *params)
         ucol_setStrength(coll, strengths[s]);
 
         // TODO: try alternate prefix and suffix too?
-        // TODO: alterntaes are only equal at primary strength. Is this OK?
+        // TODO: alternates are only equal at primary strength. Is this OK?
         for(int32_t t = 0; t < loopCount; t += 1) {
             uint32_t seed = m_seed;
             // int32_t  nmc = 0;
